//
//  ChatView.swift
//  Chat
//
//  Created by Sergey Tarasov on 24.07.2022.
//

import SwiftUI
import XMPPFramework
import XMPPFrameworkSwift

extension CGFloat {
    static let sidePadding: CGFloat = 14
}

struct ChatView: View, KeyboardReadable {

    @StateObject var viewModel: ChatViewModel

    @State private var isKeyboardVisible = false

    var body: some View {
        VStack(alignment: .center, spacing: 0) {
            ScrollView {
                ScrollViewReader { value in
                    VStack {
                        withAnimation {
							ForEach(viewModel.messages, id: \.id) { message in
								MessageView(
									text: message.body,
									time: "",
									isSelf: message.fromID == XMPPController.shared.xmppStream.myJID?.bare
								)
                            }
                            .onChange(of: viewModel.messages) { _ in
								DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
									withAnimation {
										value.scrollTo(viewModel.messages.count - 1, anchor: .bottomTrailing)
									}
								}
                            }
                            .onReceive(keyboardPublisher) { newIsKeyboardVisible in
                                isKeyboardVisible = newIsKeyboardVisible
                                withAnimation {
                                    value.scrollTo(viewModel.messages.count - 2, anchor: .topTrailing)
                                }
                            }
                            .onAppear {
                                value.scrollTo(viewModel.messages.count - 1, anchor: .top)
                            }
                        }
                    }
                    .padding(.horizontal, .sidePadding)
                    .padding(.vertical, 12)
                    .background(Color.indigo.opacity(0.0))
                }
            }
            Divider()
            HStack {
                TextField("Сообщение", text: $viewModel.messageText)
                    .textFieldStyle(.automatic)
                    .padding(.vertical, 6)
                    .padding(.horizontal, 12)
                    .background(.background)
                Button {
                    viewModel.sendMessage()
                } label: {
                    Image(systemName: "arrow.up.circle.fill")
                        .font(.title)
                        .foregroundColor(viewModel.messageText.isEmpty ? .gray : .accentColor)
                }
                .disabled(viewModel.messageText.isEmpty)
            }
            .padding(.horizontal, .sidePadding)
            .padding(.vertical, 6)
            .background()
        }
        .navigationTitle(viewModel.room.roomJID.user ?? "Room")
        .navigationBarTitleDisplayMode(.inline)
        .toolbar {
            ToolbarItem(placement: .navigationBarTrailing) {
                Button {
                    viewModel.fetchMessages()
                } label: {
                    Image(systemName: "arrow.counterclockwise")
                }
            }
        }
        .onAppear {
            viewModel.fetchMessages()
        }
    }
}

struct ChatView_Previews: PreviewProvider {
    static var viewModel: ChatViewModel = ChatViewModel(with: "")
    static var previews: some View {
        Group {
            NavigationView {
                ChatView(viewModel: viewModel)
            }
            NavigationView {
                ChatView(viewModel: viewModel)
                    .preferredColorScheme(.dark)
            }
        }
    }
}
